use strict;
use warnings;

sub gen_filtered_ptles {

    # Purpose: wrapper program for generating filtered particle files
    # from the unfiltered particle files.
    # 
    # Input arguments:
    #   $ptlefiles_ref - reference to array of ptle files
    #   $auto_ref - reference to AUTO3DEM parameter hash
    #   $nshift - number of deltas to shift cutoff by
    #
    # Return values:
    #   @selectfiles - array of selected particle files

    my @selectfiles = ();

    my ($ptlefiles_ref, $auto_ref, $nshift) = @_;
    foreach my $ptlefile (@{$ptlefiles_ref}) {
	unless(-r $ptlefile) {
	    print OUT "NOTE - unable to open $ptlefile for filtering\n";
	    print OUT "\n";
	    next;
	}
	my $selectfile = "${ptlefile}_sel";
	select_ptles($ptlefile, $selectfile, $auto_ref, $nshift);
	push(@selectfiles, $selectfile);
    }
	
    return @selectfiles;
}


sub select_ptles {

    # Purpose: select particles from a parameter file on the basis of
    # orientation (theta and phi), distance between center of particle
    # and center of box, and correlation coefficients (SEARCH mode) or
    # score (REFINE mode). These filters can be combined.
    #
    # When filtering on the correlation coefficients, can specify:
    # (1) Fraction of particles to be selected
    # (2) Number of particles to be selected
    # (3) A cutoff of (avg + N * std_dev) to be used
    #
    # The selection of particles is tweaked by nshift*delta, with the
    # usage dependent on the context
    #
    # (1) Fraction of ptles - shifts the fraction selected
    # (2) Number of ptles - shifts number selected
    # (3) Number of stddevs - shifts N*std_dev
    #
    # Input arguments:
    #   $ptlefile - name of parameter file
    #   $selectfile - name of new file of filtered particles
    #   $auto_ref - reference to AUTO3DEM parameter hash
    #   $nshift - number of deltas to shift cutoff by
    #
    # Return values:
    #   None

    my($ptlefile, $selectfile, $auto_ref, $nshift) = @_;

    my @record;
    my @flag;
    my @pft_cc = ();
    my @prj_cc = ();
    my @cmp_cc = ();
    my ($pft_cc_sum,  $prj_cc_sum,  $cmp_cc_sum);
    my ($pft_cc2_sum, $prj_cc2_sum, $cmp_cc2_sum);
    my $nptles;

    ####### Scan through through file to ...
    # (1) Calculate sums of correlation coefficients
    # (2) Set filters based on theta and phi
    # (3) Set filters based on distance from center of box
    open(PTLE, $ptlefile) || die
	"Unable to open file $ptlefile for reading in select_ptles\n";
    while(<PTLE>) {

	### By default, record is accepted
	$flag[$.] = 1;
	$record[$.] = $_;

	### Skip over the header lines
	if($. == 1 || $. == 2) {
	    next;
	}

	### Get rid of whitespace and split line into fields
	s/\s+//g;
	my ($id, $theta, $phi, $omega, 
	    $xcent, $ycent, $mag, $pft_cc,
	    $prj_cc, $cmp_cc) = split(/,/);

	### Fill arrays of correlation coeffficients
	### Calculate sums and sums squared
	unless($pft_cc =~ /\d/) {
	    $pft_cc = 0;
	}
	unless($prj_cc =~ /\d/) {
	    $prj_cc = 0;
	}
	unless($cmp_cc =~ /\d/) {
	    $cmp_cc = 0;
	}
	push(@pft_cc, $pft_cc);
	push(@prj_cc, $prj_cc);
	push(@cmp_cc, $cmp_cc);
	$pft_cc_sum  += $pft_cc;
	$pft_cc2_sum += $pft_cc*$pft_cc;
	$prj_cc_sum  += $prj_cc;
	$prj_cc2_sum += $prj_cc*$prj_cc;
	$cmp_cc_sum  += $cmp_cc;
	$cmp_cc2_sum += $cmp_cc*$cmp_cc;
	$nptles++;

	### Test theta and phi
	if($theta >= $$auto_ref{theta_reject_lower} &&
	   $theta <= $$auto_ref{theta_reject_upper} &&
	   $phi   >= $$auto_ref{phi_reject_lower}   &&
	   $phi   <= $$auto_ref{phi_reject_upper}) {
	    $flag[$.] = 0;
	}

	### Test center of particle relative to box center
	if($xcent < $$auto_ref{boxrad}-$$auto_ref{box_center_offset} ||
	   $xcent > $$auto_ref{boxrad}+$$auto_ref{box_center_offset} ||
	   $ycent < $$auto_ref{boxrad}-$$auto_ref{box_center_offset} ||
	   $ycent > $$auto_ref{boxrad}+$$auto_ref{box_center_offset}) {
	    $flag[$.] = 0;
	}
	
    }
    close(PTLE);

    ####### Calculate averages and standard deviations of CC's
    my ($pft_cc_avg, $prj_cc_avg, $cmp_cc_avg) = (0, 0, 0);
    my ($pft_cc_std, $prj_cc_std, $cmp_cc_std) = (0, 0, 0);

    if($nptles > 0) {
	$pft_cc_avg = $pft_cc_sum/$nptles;
    }
    if($nptles > 1 && ($pft_cc2_sum - $nptles*$pft_cc_avg**2) > 0) {
	$pft_cc_std = sqrt(($pft_cc2_sum - $nptles*$pft_cc_avg**2)/
			  ($nptles-1));
    }

    if($nptles > 0) {
	$prj_cc_avg = $prj_cc_sum/$nptles;
    }
    if($nptles > 1 && ($prj_cc2_sum - $nptles*$prj_cc_avg**2) > 0) {
	$prj_cc_std = sqrt(($prj_cc2_sum - $nptles*$prj_cc_avg**2)/
			  ($nptles-1));
    }

    if($nptles > 0) {
	$cmp_cc_avg = $cmp_cc_sum/$nptles;
    }
    if($nptles > 1 && ($cmp_cc2_sum - $nptles*$cmp_cc_avg**2) > 0) {
	$cmp_cc_std = sqrt(($cmp_cc2_sum - $nptles*$cmp_cc_avg**2)/
			  ($nptles-1));
    }

    print OUT "Filtering $ptlefile (selection criterion = $nshift)\n";
    print OUT "   Correlation coefficient statistics\n";
    print OUT "   pft_cc/score avg:                $pft_cc_avg\n";
    print OUT "   pft_cc/score stddev:             $pft_cc_std\n";
    print OUT "   prj_cc avg:                      $prj_cc_avg\n";
    print OUT "   prj_cc stddev:                   $prj_cc_std\n";
    print OUT "   cmp_cc avg:                      $cmp_cc_avg\n";
    print OUT "   cmp_cc stddev:                   $cmp_cc_std\n";
    print OUT "\n";
    
    ####### When we're running PO2R, the three "correlation coefficients"
    # have a different meaning. The pft_cc column is a score that we
    # want to minimize; the prj_cc and cmp_cc columns are undefined.
    my ($score_avg, $score_std) = ($pft_cc_avg, $pft_cc_std);
    my @score = @pft_cc;

    ####### Get the cutoffs for correlation coefficients or score
    my ($pft_cc_cutoff, $prj_cc_cutoff, $cmp_cc_cutoff, $score_cutoff);

    if(defined $$auto_ref{score_fraction}) {
	my $nselect;
	@score = sort numerically @score;
	if($$auto_ref{score_fraction} <= 1) {
	    my $fraction = $$auto_ref{score_fraction} + 
		$nshift*$$auto_ref{select_delta};
	    if($fraction > 1) {
		$fraction = 1;
	    } elsif ($fraction < 0) {
		$fraction = 0;
	    }
	    $nselect = $nptles * $fraction;
	} else {
	    $nselect = $$auto_ref{score_fraction} +
		$nshift*$$auto_ref{select_delta};
	    if($nselect > $nptles) {
		$nselect = $nptles;
	    } else {
		$nselect = 0;
	    }
	}
	$score_cutoff = $score[$nselect - 1];
    }

    if(defined $$auto_ref{pft_cc_fraction}) {
	my $nselect;
	@pft_cc = reverse sort numerically @pft_cc;
	if($$auto_ref{pft_cc_fraction} <= 1) {
	    my $fraction = $$auto_ref{pft_cc_fraction} + 
		$nshift*$$auto_ref{select_delta};
	    if($fraction > 1) {
		$fraction = 1;
	    } elsif ($fraction < 0) {
		$fraction = 0;
	    }
	    $nselect = $nptles * $fraction;
	} else {
	    $nselect = $$auto_ref{pft_cc_fraction} +
		$nshift*$$auto_ref{select_delta};
	    if($nselect > $nptles) {
		$nselect = $nptles;
	    } else {
		$nselect = 0;
	    }
	}
	$pft_cc_cutoff = $pft_cc[$nselect - 1];
    }

    if(defined $$auto_ref{prj_cc_fraction}) {
	my $nselect;
	@prj_cc = reverse sort numerically @prj_cc;
	if($$auto_ref{prj_cc_fraction} <= 1) {
	    my $fraction = $$auto_ref{prj_cc_fraction} + 
		$nshift*$$auto_ref{select_delta};
	    if($fraction > 1) {
		$fraction = 1;
	    } elsif ($fraction < 0) {
		$fraction = 0;
	    }
	    $nselect = $nptles * $fraction;
	} else {
	    $nselect = $$auto_ref{prj_cc_fraction} +
		$nshift*$$auto_ref{select_delta};
	    if($nselect > $nptles) {
		$nselect = $nptles;
	    } else {
		$nselect = 0;
	    }
	}
	$prj_cc_cutoff = $prj_cc[$nselect - 1];
    }

    if(defined $$auto_ref{cmp_cc_fraction}) {
	my $nselect;
	@cmp_cc = reverse sort numerically @cmp_cc;
	if($$auto_ref{cmp_cc_fraction} <= 1) {
	    my $fraction = $$auto_ref{cmp_cc_fraction} + 
		$nshift*$$auto_ref{select_delta};
	    if($fraction > 1) {
		$fraction = 1;
	    } elsif ($fraction < 0) {
		$fraction = 0;
	    }
	    $nselect = $nptles * $fraction;
	} else {
	    $nselect = $$auto_ref{cmp_cc_fraction} +
		$nshift*$$auto_ref{select_delta};
	    if($nselect > $nptles) {
		$nselect = $nptles;
	    } else {
		$nselect = 0;
	    }
	}
	$cmp_cc_cutoff = $cmp_cc[$nselect - 1];
    }

    if(defined $$auto_ref{score_nstd}) {
	$score_cutoff = $score_avg + 
	    ($$auto_ref{score_nstd}+$$auto_ref{select_delta})*$score_std;
    }

    if(defined $$auto_ref{pft_cc_nstd}) {
	$pft_cc_cutoff = $pft_cc_avg + 
	    ($$auto_ref{pft_cc_nstd}+$$auto_ref{select_delta})*$pft_cc_std;
    }

    if(defined $$auto_ref{prj_cc_nstd}) {
	$prj_cc_cutoff = $prj_cc_avg + 
	    ($$auto_ref{prj_cc_nstd}+$$auto_ref{select_delta})*$prj_cc_std;
    }

    if(defined $$auto_ref{cmp_cc_nstd}) {
	$cmp_cc_cutoff = $cmp_cc_avg + 
	    ($$auto_ref{cmp_cc_nstd}+$$auto_ref{select_delta})*$cmp_cc_std;
    }

    if(defined $score_cutoff) {
	print OUT "   score_cutoff:                    $score_cutoff\n";
    }
    if(defined $pft_cc_cutoff) {
	print OUT "   pft_cc_cutoff:                   $pft_cc_cutoff\n";
    }
    if(defined $prj_cc_cutoff) {
	print OUT "   prj_cc_cutoff:                   $prj_cc_cutoff\n";
    }
    if(defined $cmp_cc_cutoff) {
	print OUT "cmp_cc_cutoff:                      $cmp_cc_cutoff\n";
    }
    print OUT "\n";

    ####### Determine whether or not this was a PFTsearch generated file
    # For PO2R generated files, the prj_cc and cmp_cc will be zero
    my $pftsearch_gen;
    if($prj_cc_avg != 0 && $cmp_cc_avg != 0) {
	$pftsearch_gen = 1;
    } else {
	$pftsearch_gen = 0;
    }

    ####### Test CC's and print out new file of selected particles
    open(SEL, ">$selectfile") || die
	"Unable to open $selectfile for writing in select_ptles\n";
    print SEL $record[1]; # Header line
    print SEL $record[2]; # Header line

    my $imax = @flag-1;
    for (my $i=3; $i<=$imax; $i++) {

	# Split record to get correlation coefficients
	my ($id, $theta, $phi, $omega, 
	    $xcent, $ycent, $mag, $pft_cc,
	    $prj_cc, $cmp_cc) = split(/,/, $record[$i]);

	# Apply the cutoff on pft_cc only for PFT generated files
	if(defined $pft_cc_cutoff && $pftsearch_gen) {
	    if($pft_cc < $pft_cc_cutoff) {
		$flag[$i] = 0;
	    }
	}

	# Apply the cutoff on prj_cc only for PFT generated files
	if(defined $prj_cc_cutoff && $pftsearch_gen) {
	    if($prj_cc < $prj_cc_cutoff) {
		$flag[$i] = 0;
	    }
	}

	# Apply the cutoff on cmp_cc only for PFT generated files
	if(defined $cmp_cc_cutoff && $pftsearch_gen) {
	    if($cmp_cc < $cmp_cc_cutoff) {
		$flag[$i] = 0;
	    }
	}

	# Apply the cutoff on score only for PO2R generated files
	if(defined $score_cutoff && !$pftsearch_gen) {
	    if($pft_cc > $score_cutoff) {
		$flag[$i] = 0;
	    }
	}

	if($flag[$i] > 0) {
	    print SEL $record[$i];
	}
    }
    close(SEL);

    return;
}

sub numerically {
    $a <=> $b;
}
1;
