      subroutine setrotmat(angles_rad, rot_mat) 
C
C     Author: Robert E. Lynch (rel@cs.purdue.edu)
C     Date:   1997
C
C     Function: Form matrix which rotates pixel plane to standard
C     coordinate system. To be consistent with usage in POR, P3DR, and
C     PCTFR, the INVERSE of the rotation matrix is returned.
C
C     Usage: call setrotmat( angles_rad, rot_mat)
C
C     Input: angles_rad - Rotation angles THETA, PHI, OMEGA in radians.
C
C     Output: rot_mat (Real) -- Rotation matrix which maps a point
C     (U,V,W)^T in the space of pixel Fourier coefficients to the
C     corresponding point (X,Y,Z)^T = ROT_MAT(U,V,W)^T in the standard
C     cartesian space of Fourier coefficients of the particle.
C
C     Algorithm:
C     Matrix which rotates a vector about the original y-axis:
C
C             |   cos(theta)             sin(theta) |
C     rot_1 = |    1            1          0        |
C             |  -sin(theta)             cos(theta) |
C
C     Matrix which rotates a vector about the original z-axis:
C
C             |  cos(phi)  -sin(phi)       0        |
C     rot_2 = |  sin(phi)   cos(phi)       0        |
C             |    0          0            1        |
C
C     The original x, y, z axes are now vectors x_21, y_21, z_21:
C
C                     | 1 0 0 |   |                |
C     rot_2 * rot_1 * | 0 1 0 | = | x_21 y_21 z_21 |
C                     | 0 0 1 |   |                |
C
C     i.e, the original x-axis = ( 1, 0, 0)^T is now the vector
C     x_21 where x_21 = rot_2 * rot_1 * ( 1, 0, 0 )
C
C     Rotation about the new z axis, which is the vector v_z, is
C     obtained by
C     x_21 --> x_213 =  cos(omega) * x_21 + sin(omega) * y_21
C     y_21 --> y_213 = -sin(omega) * x_21 + cos(omega) * y_21
C     z_21 --> z_213
C
C     A vector expressed with respect to the axes x, y, z is
C     transformated by the rotations is accomplished by multiplication
C     of the vector by the matrix rot = rot_2 * rot_1 * rot_3 where
C
C             |  cos(omega)  -sin(omega)       0   |
C     rot_3 = |  sin(omega)   cos(omega)       0   |
C             |     0            0             1   |
C
C     Thus, ROT_MAT = ROT_2 * ROT_1 * ROT_3
C
C     Modification history:                                         
C     - Hong Lin, Version 1.0, Aug.18, 1998
C     - Hong Lin, Version 2.0, Oct.11, 1999
C     - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002
C     - R Sinkovits, 6/30/06, Major cleanup and modernization

      implicit none
      
      real, intent(in)  :: angles_rad(1:3)
      real, intent(out) :: rot_mat(1:3,1:3)
      
      real rot_1(3,3), rot_2(3,3), rot_3(3,3), 
     +     theta_rad, phi_rad, omega_rad
      
      theta_rad  = angles_rad(1)
      phi_rad    = angles_rad(2)
      omega_rad  = angles_rad(3)
      
      rot_1(1,1) =   cos( theta_rad )
      rot_1(1,2) =   0.
      rot_1(1,3) =   sin( theta_rad )
      rot_1(2,1) =   0.
      rot_1(2,2) =   1.
      rot_1(2,3) =   0.
      rot_1(3,1) = - rot_1(1,3)
      rot_1(3,2) =   0.
      rot_1(3,3) =   rot_1(1,1)
      
      rot_2(1,1) =   cos( phi_rad )
      rot_2(1,2) = - sin( phi_rad )
      rot_2(1,3) =   0.
      rot_2(2,1) = - rot_2(1,2)
      rot_2(2,2) =   rot_2(1,1)
      rot_2(2,3) =   0.
      rot_2(3,1) =   0.
      rot_2(3,2) =   0.
      rot_2(3,3) =   1.
      
      rot_3(1,1) =   cos( omega_rad )
      rot_3(1,2) = - sin( omega_rad )
      rot_3(1,3) =   0.
      rot_3(2,1) = - rot_3(1,2)
      rot_3(2,2) =   rot_3(1,1)
      rot_3(2,3) =   0.
      rot_3(3,1) =   0.
      rot_3(3,2) =   0.
      rot_3(3,3) =   1.
      
C     Form product of three matrices ROT_2 * ROT_1 * ROT_3
      rot_mat = matmul(matmul(rot_2, rot_1), rot_3)

C     Calculate inverse of rotation matrix. Since ROT_MAT is orthogonal,
C     this is equivalent to taking the transpose
      rot_mat = transpose(rot_mat)
      
      return
      end subroutine setrotmat
